"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const input_stream_1 = require("./input-stream");
function TokenStream(input, separator = ',') {
    let current = null;
    return { next, peek, eof, croak: input.croak };
    function isWhitespace(char) {
        return (separator === '\t' ? ' ' : ' \t').indexOf(char) >= 0;
    }
    function readWhile(predicate) {
        let str = '';
        while (!input.eof() && predicate(input.peek()))
            str += input.next();
        return str;
    }
    function readWrapped() {
        let str = '';
        input.next();
        while (!input.eof()) {
            const ch = input.next();
            if (ch === '"' && input.peek() === '"') {
                str += '"';
                input.next();
            }
            else if (ch === '"') {
                break;
            }
            else {
                str += ch;
            }
        }
        return str;
    }
    function readUnwrapped() {
        let str = '';
        while (!input.eof()) {
            const ch = input.next();
            if (ch === '"')
                input.croak('qoute must be wrapped by qoute');
            str += ch;
            if (`${separator}\n`.includes(input.peek()))
                break;
        }
        return str.trim();
    }
    function readString(wraped) {
        return { type: 'item', value: wraped ? readWrapped() : readUnwrapped() };
    }
    function readNext() {
        readWhile(isWhitespace);
        if (input.eof())
            return null;
        const ch = input.peek();
        if (ch === '"')
            return readString(true);
        if (ch === separator)
            return { type: 'comma', value: input.next() };
        if (ch === '\n')
            return { type: 'newline', value: input.next() };
        return readString(false);
    }
    function peek() {
        return current || (current = readNext());
    }
    function next() {
        const tok = current;
        current = null;
        return tok || readNext();
    }
    function eof() {
        return peek() === null;
    }
}
function Parse(input) {
    const data = [];
    function readWhileNewLine() {
        while (!input.eof() && input.peek().type === 'newline') {
            input.next();
        }
    }
    function readItems() {
        const list = [];
        while (!input.eof() && input.peek().type !== 'newline') {
            list.push(input.next());
        }
        return list;
    }
    while (!input.eof()) {
        readWhileNewLine();
        const item = readItems();
        if (item.length > 0)
            data.push(item);
    }
    return data;
}
function toString(item) {
    const qoute = '"';
    if (typeof item === 'string' && /[\n,"]/g.test(item)) {
        return `"${item.replace(qoute, qoute.repeat(2))}"`;
    }
    return item;
}
/**
 * 读取并解析CSV字符串
 * @param input - csv 字符串
 * @param delimiter - 分隔符 默认值是 `,`
 * @public
 */
function parse(input, delimiter = ',') {
    const list = Parse(TokenStream(input_stream_1.InputStream(input), delimiter));
    return list.map((row) => {
        const data = [];
        let hasItem = false;
        for (const item of row) {
            if (item !== null) {
                if (item.type === 'item') {
                    hasItem = true;
                    data.push(item.value);
                }
                else if (item.type === 'comma') {
                    if (hasItem)
                        hasItem = false;
                    else
                        data.push('');
                }
            }
        }
        return data;
    });
}
exports.parse = parse;
/**
 * 生成CSV字符串
 * @param input - 数据(二维数组)
 * @public
 */
function stringify(input, separator = ',') {
    return input.map((row) => row.map(toString).join(separator)).join('\n');
}
exports.stringify = stringify;
//# sourceMappingURL=index.js.map